package com.dxhy.common.datasource.config;

import java.util.ArrayDeque;
import java.util.Deque;

import org.apache.commons.lang3.StringUtils;

import lombok.extern.slf4j.Slf4j;

/**
 * 多数据源上下文
 *
 * @author jiaohongyang
 */
@Slf4j

public class DynamicContextHolder {
    private static final ThreadLocal<Deque<String>> CONTEXT_HOLDER = ThreadLocal.withInitial(() -> new ArrayDeque());

    /**
     * 获得当前线程数据源
     *
     * 数据源名称
     */
    public static String peek() {
        return CONTEXT_HOLDER.get().peek();
    }

    /**
     * 设置当前线程数据源
     *
     * @param dataSource
     *            数据源名称
     */
    public static void push(String dataSource) {
        DynamicContextHolder.poll();
        String s = dataSource.split("_")[0];
        if (StringUtils.isNotBlank(s)) {
            log.debug("获取当前数据源：" + dataSource);
            CONTEXT_HOLDER.get().push(dataSource);
        } else {
            log.debug("获取当前数据源：" + dataSource);
            CONTEXT_HOLDER.get().push("business" + dataSource);
        }

    }

    /**
     * 清空当前线程数据源
     */
    public static void poll() {
        Deque<String> deque = CONTEXT_HOLDER.get();
        deque.poll();
        if (deque.isEmpty()) {
            CONTEXT_HOLDER.remove();
        }
    }

}
