package com.dxhy.core.job.entity;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonProperty;

import lombok.Data;

/**
 * 系统用户
 *
 */
@Data
@TableName("sys_user")
public class SysUserEntity implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * 用户ID
     */
    @TableId
    private Long userId;

    /**
     * 用户名
     */
    @NotBlank(message = "用户名不能为空")
    private String username;

    private String loginname;
    /**
     * 密码
     */
    @NotBlank(message = "密码不能为空")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private String password;

    /**
     * 盐
     */
    private String salt;

    /**
     * 邮箱
     */
    @NotBlank(message = "邮箱不能为空")
    @Email(message = "邮箱格式不正确")
    private String email;

    /**
     * 手机号
     */
    private String mobile;

    /**
     * 状态 0：禁用 1：正常
     */
    private Integer status;
    /**
     * 明文密码
     */
    private String plpassword;

    /**
     * 角色ID列表
     */
    @TableField(exist = false)
    private List<Long> roleIdList;

    @TableField(exist = false)
    private List<Long> businessIdList;

    /**
     * 创建时间
     */
    private Date createTime;

    /**
     * 部门ID
     */
    @NotNull(message = "部门不能为空")
    private Long deptId;

    /**
     * 用户类型 1--购方用户 2--供应商
     */
    private String userType;

    /**
     * 销方税号
     */
    private String taxno;
    /**
     * 供应商编码
     */
    private String userCode;

    /**
     * 部门名称
     */
    @TableField(exist = false)
    private String deptName;

    private String company;

    @TableField(exist = false)
    private String qsStatus;

    // 专票限额
    private String zpAmount;
    // 普票限额
    private String ppAmount;
    // 开票类型 01--专票 04 --普票 00 -- 即开专票又开普票
    private String kpType;
    // 开票人
    private String kpUser;
    // 收款人
    private String skUser;
    // 复核人
    private String fhUser;
    // 税盘类型 1--金税盘 2--税控盘,3--ukey
    private String spType;
    // 税盘编号
    private String spCode;

    private String contactUser;// 联系人
    private String certPassword;// 数字证书密码
    private String taxPassword;// 税盘口令
    private String authorizeCode;// 注册码
    private String message;
    private Date lastDate;// 截止时间
    private String signatureId;
    private String fpzyztp;// 发票专用章图片
}
