package com.dxhy.core.thread;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.SignQueryVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.MryxFpqsService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * 每日优鲜扫描明细导出线程
 *
 * @author user
 */

@SuppressWarnings({"AlibabaMethodTooLong", "AlibabaLowerCamelCaseVariableNaming", "AlibabaClassNamingShouldBeCamel"})
@Slf4j

public class MRYXqsExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private String fileNamePrefix = "invoiceQsListMRYX";
    /**
     * 文件名后缀
     */
    private String fileNameSurfix = ".xlsx";
    /**
     * 文件名中间分隔符
     */
    private String fileSplit = "_";
    /**
     * excel的标题
     */
    private String title = "发票扫描明细";

    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "扫描备注", "发票代码", "发票号码", "发票类型", "开票日期", "购方税号", "销方税号", "金额", "税额",
        "价税合计", "扫描日期", "签收日期", "签收状态", "签收人"};

    private IExcelExportlogService exportLogService;

    private List<TDxExcelExportlog> exportLogs;

    private MryxFpqsService fpqsService;

    private FileService fileService;

    private Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public MRYXqsExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        MryxFpqsService fpqsService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService) {
        this.tabulateRedisService = tabulateRedisService;
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.fpqsService = fpqsService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
    }

    private IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();

            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfsh = prams.getString("gfsh");
                String qsrqq = prams.getString("qsrqq");
                String qsrqz = prams.getString("qsrqz");
                String invoiceType = prams.getString("invoiceType");
                String invoiceSource = prams.getString("invoiceSource");
                String invoiceNo = prams.getString("invoiceNo");
                String invoiceCode = prams.getString("invoiceCode");
                String qsStatus = prams.getString("qsStatus");
                String gfshFlag = prams.getString("gfshFlag");
                String createDateBegin = prams.getString("createDateBegin");
                String createDateEnd = prams.getString("createDateEnd");
                String qsNameFlag = prams.getString("qsNameFlag");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);
                // 参数转换为需要的形式
                Map<String, Object> pramsMap = new HashMap<String, Object>(16);

                pramsMap.put("userName", prams.getString("userName"));
                pramsMap.put("dbName", dbName);

                List<String> gfshList = new ArrayList<String>();
                if (!StringHelper.isBlank(gfsh)) {
                    gfshList = StringHelper.stringToList(gfsh);
                    pramsMap.put("gfsh", gfshList);
                    pramsMap.put("gfshFlag", gfshFlag);
                } else {
                    pramsMap.put("gfsh", gfshList);
                }

                pramsMap.put("invoiceCode", invoiceCode);

                if (!StringHelper.isBlank(qsrqq)) {

                    pramsMap.put("qsrqq", DateUtils.getMorning(DateUtils.strToDate(qsrqq)));
                }
                if (!StringHelper.isBlank(qsrqz)) {

                    pramsMap.put("qsrqz", DateUtils.getNight(DateUtils.strToDate(qsrqz)));
                }

                if (!StringHelper.isBlank(createDateBegin)) {

                    pramsMap.put("createDateBegin", DateUtils.getMorning(DateUtils.strToDate(createDateBegin)));
                }

                if (!StringHelper.isBlank(createDateEnd)) {

                    pramsMap.put("createDateEnd", DateUtils.getNight(DateUtils.strToDate(createDateEnd)));

                }

                if (!StringHelper.isBlank(prams.getString("invoiceDateBegin"))) {

                    pramsMap.put("invoiceDateBegin",
                        DateUtils.getMorning(DateUtils.strToDate(prams.getString("invoiceDateBegin"))));
                }

                if (!StringHelper.isBlank(prams.getString("invoiceDateEnd"))) {

                    pramsMap.put("invoiceDateEnd",
                        DateUtils.getNight(DateUtils.strToDate(prams.getString("invoiceDateEnd"))));
                }

                if (!StringHelper.isBlank(invoiceType) && !"99".equals(invoiceType)) {
                    pramsMap.put("invoiceType", invoiceType);
                }
                if (!StringHelper.isBlank(invoiceSource) && !"99".equals(invoiceSource)) {
                    pramsMap.put("invoiceSource", invoiceSource);
                }
                if (!StringHelper.isBlank(invoiceNo)) {
                    pramsMap.put("invoiceNo", invoiceNo);
                }

                if (!StringHelper.isBlank(qsStatus)) {
                    if (!"99".equals(qsStatus)) {
                        pramsMap.put("qsStatus", String.valueOf(Integer.parseInt(qsStatus)));
                    }
                }
                if (StringUtils.isNotBlank(qsNameFlag)) {
                    pramsMap.put("qsNameFlag", qsNameFlag);
                }
                pramsMap.put("company", company);
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(pramsMap, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = fpqsService.selectQsfpList(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<SignQueryVO> dataList = (List<SignQueryVO>)pageData.get("datalist");

        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return hasNext;
        }
    }
}
