package com.dxhy.core.thread;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.MixedSignService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 查验历史明细导出
 *
 * @author user
 */
@SuppressWarnings({"AlibabaMethodTooLong", "DuplicatedCode", "FieldCanBeLocal"})
@Slf4j

public class MixedSignThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "mixedSign";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xlsx";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "扫描确认";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "扫描描述", "单据类型", "发票类型", "签收状态", "确认状态", "扫描日期", "发票代码", "发票号码", "开票日期",
        "购方名称", "购方税号", "销方名称", "销方税号", "金额", "税额", "价税合计", "车次/航班号", "出发城市", "到达城市", "乘坐人", "校验码", "备注", "业务单元"};

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final MixedSignService mixedSignService;

    private final FileService fileService;

    private final Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public MixedSignThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        MixedSignService mixedSignService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService) {
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.mixedSignService = mixedSignService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserName();
            String userName = exportLog.getUserName();

            // 导出条件
            String condition = exportLog.getConditions();

            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);

                String gfsh = prams.getString("gfsh");
                String invoiceType = prams.getString("invoiceType");
                String fileType = prams.getString("fileType");
                String businessCode = prams.getString("businessCode");
                String createDateBegin = prams.getString("createDateBegin");
                String createDateEnd = prams.getString("createDateEnd");
                String xfName = prams.getString("xfName");
                String ewm = prams.getString("ewm");
                String qzStatus = prams.getString("qzStatus");
                String qsStatus = prams.getString("qsStatus");
                String company = prams.getString("company");
                String gfshFlag = prams.getString("gfshFlag");
                Map<String, Object> pramsMap = new HashMap<>(16);
                pramsMap.put("dbName", dbName);
                pramsMap.put("userid", userid);
                pramsMap.put("userName", userName);
                pramsMap.put("gfshFlag", gfshFlag);
                if (StringUtils.isNotBlank(gfsh) && !gfsh.contains(",")) {
                    pramsMap.put("gfsh", gfsh);
                }

                if (!StringHelper.isBlank(invoiceType) && !"99".equals(invoiceType)) {
                    pramsMap.put("invoiceType", invoiceType);
                }

                if (StringUtils.isNotBlank(fileType)) {
                    pramsMap.put("fileType", fileType);
                }

                if (StringUtils.isNotBlank(businessCode)) {
                    pramsMap.put("businessCode", businessCode);
                }
                if (StringUtils.isNotBlank(createDateBegin)) {
                    pramsMap.put("createDateBegin", createDateBegin);
                }
                if (StringUtils.isNotBlank(createDateEnd)) {
                    pramsMap.put("createDateEnd", createDateEnd + " 23:59:59");
                }
                if (StringUtils.isNotBlank(xfName)) {
                    pramsMap.put("xfName", xfName);
                }
                if (StringUtils.isNotBlank(ewm)) {
                    pramsMap.put("ewm", ewm);
                }
                if (StringUtils.isNotBlank(qzStatus)) {
                    pramsMap.put("qzStatus", qzStatus);
                }
                if (StringUtils.isNotBlank(qsStatus)) {
                    pramsMap.put("qsStatus", qsStatus);
                }
                pramsMap.put("company", company);

                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(pramsMap, default_curr, excelFile.toString());

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *             异常
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile) throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = mixedSignService.getMixedSignList(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<String[]> dataList = (List<String[]>)pageData.get("datalist");

        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel(excelFile, title, titleColumns, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel(excelFile, title, titleColumns, dataList, firstPage, true);
            return false;
        }
    }
}
