package com.dxhy.core.thread;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.TicketService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 实名客票
 *
 * @author user
 */
@SuppressWarnings({"AlibabaUndefineMagicConstant", "AlibabaMethodTooLong", "DuplicatedCode", "FieldCanBeLocal"})
@Slf4j

public class QypcsmkpExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "实名客票";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xlsx";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private String title = "实名客票";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "发票类型", "发票代码", "发票号码", "开票日期", "销方名称", "乘坐人", "证件号", "班次/车次", "金额",
        "税率(%)", "税额", "签收状态", "签收日期", "签收方式", "签收人", "抵扣状态", "业务单号", "入池时间","凭证号", "过账时间", "影像号", "账期"};

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final TicketService ticketService;

    private final FileService fileService;

    private final Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public QypcsmkpExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        TicketService ticketService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService) {
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.ticketService = ticketService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();
            // 导出条件

            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }

            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                prams.put("dbName", dbName);
                // 参数转换为需要的形式
                String gfTaxNo = prams.getString("gfTaxNo");
                String gfsh = prams.getString("gfsh");
                List<String> gfshList = new ArrayList<>();
                if (!StringHelper.isBlank(gfsh)) {
                    gfshList = StringHelper.stringToList(gfsh);
                }
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(DateUtils.dateToMonthStr(new Date()));
                ftpFileName.append(fileSplit);
                if ("99".equals(gfTaxNo)) {
                    if (gfshList.size() > 0) {
                        ftpFileName.append(ticketService.getblocName(gfshList.get(0)));
                        ftpFileName.append(fileSplit);
                    }
                } else {
                    if (gfshList.size() > 0) {
                        for (String s : gfshList) {
                            ftpFileName.append(ticketService.getGfName(s));
                            ftpFileName.append(fileSplit);
                        }
                    }
                }

                ftpFileName.append(DateUtils.getNowTimeStamp());
                title = ftpFileName.toString();
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(prams, default_curr, excelFile.toString());

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param paramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *             异常
     *
     */

    private boolean cicleExport(Map<String, Object> paramsMap, int curr, String excelFile) throws Exception {
        String gfsh = paramsMap.get("gfsh") + "";
        List<String> gfshList = new ArrayList<>();
        List<String> invoiceTypeList = new ArrayList<>();
        if (!StringHelper.isBlank(gfsh)) {
            gfshList = StringHelper.stringToList(gfsh);
            paramsMap.put("gfsh", gfshList);
        } else {
            paramsMap.put("gfsh", gfshList);
        }

        if (paramsMap.get("invoiceType") != null && !"".equals(paramsMap.get("invoiceType"))
            && !"99".equals(paramsMap.get("invoiceType"))) {
            String[] invoiceTypeArr = paramsMap.get("invoiceType").toString().split(",");
            for (String uuid : invoiceTypeArr) {
                invoiceTypeList.add(uuid);
                paramsMap.put("list", invoiceTypeList);
            }
        }
        if (paramsMap.get("kpksrq") != null && !"".equals(paramsMap.get("kpksrq"))) {
            paramsMap.put("kpksrq", DateUtils.getMorning(DateUtils.strToDate(paramsMap.get("kpksrq").toString())));
        }
        if (paramsMap.get("kpjsrq") != null && !"".equals(paramsMap.get("kpjsrq"))) {
            paramsMap.put("kpjsrq", DateUtils.getNight(DateUtils.strToDate(paramsMap.get("kpjsrq").toString())));
        }
        if (paramsMap.get("cjksyf") != null && !"".equals(paramsMap.get("cjksyf"))) {
            paramsMap.put("cjksyf", DateUtils.getMorning(DateUtils.strToDate(paramsMap.get("cjksyf").toString())));
        }
        if (paramsMap.get("cjjsyf") != null && !"".equals(paramsMap.get("cjjsyf"))) {
            paramsMap.put("cjjsyf", DateUtils.getNight(DateUtils.strToDate(paramsMap.get("cjjsyf").toString())));
        }

        if (paramsMap.get("qsyf") == null && "".equals(paramsMap.get("qsyf")) && !"1".equals(paramsMap.get("qszt"))) {
            paramsMap.put("qsyf", "");
        }
        if ("99".equals(paramsMap.get("isDeduction"))) {
            paramsMap.put("isDeduction", "");
        }
        if ("99".equals(paramsMap.get("invoiceType"))) {
            List<String> list = new ArrayList<>();
            list.add("90");
            list.add("89");
            list.add("92");
            list.add("10");
            list.add("87");
            list.add("88");
            paramsMap.put("list", list);
        }
        // 查询指定页数的数据
        JSONObject pageData = ticketService.selectByPramsMap(paramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<String[]> dataList = (List<String[]>)pageData.get("datalist");
        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel(excelFile, title, titleColumns, dataList, firstPage, false);
            // 更新页数
            return cicleExport(paramsMap, nextPage, excelFile);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel(excelFile, title, titleColumns, dataList, firstPage, true);
            return false;
        }

    }
}
