
package com.dxhy.customs.service;

import java.util.Map;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dxhy.common.vo.UserInfo;
import com.dxhy.customs.entity.TdxCustomsRecord;
import com.dxhy.customs.model.TdxCustomsRecordDTO;

/**
 * @author dxhy
 */
public interface CustomsRecordService {

    /**
     * 海关缴款书采集查询
     *
     * @param page
     *            分页
     * @param entity
     *            条件
     * @param userInfo
     *            用户信息
     * @return 返回
     *
     */
    String query(Page<?> page, TdxCustomsRecordDTO entity, UserInfo userInfo);

    /**
     * 海关缴款书采集明细
     *
     * @param page
     *            分页
     * @param entity
     *            条件
     * @param userInfo
     *            用户信息
     * @return 返回
     */
    String detail(Page<?> page, TdxCustomsRecordDTO entity, UserInfo userInfo);

    /**
     * 海关缴款书录入查询
     *
     * @param page
     *            分页
     * @param entity
     *            条件
     * @param userInfo
     *            用户信息
     * @return 返回
     *
     */
    String insertQuery(Page<?> page, TdxCustomsRecordDTO entity, UserInfo userInfo);

    /**
     * 海关缴款书录入保存
     *
     * @param entity
     *            海关缴款书信息你
     * @param userInfo
     *            用户信息
     * @return 返回
     *
     */
    String insert(TdxCustomsRecord entity, UserInfo userInfo);

    /**
     * 缴款书编辑
     *
     * @author 陈刚扬
     * @date 2021/2/23
     * @param entity
     * @param userInfo
     * @return java.lang.String
     */
    String editCustoms(TdxCustomsRecord entity, UserInfo userInfo) throws Exception;

    /**
     * 删除缴款书
     *
     * @author 陈刚扬
     * @date 2021/2/23
     * @param entity
     * @param userInfo
     * @return java.lang.String
     */
    String deleteCustoms(TdxCustomsRecord entity, UserInfo userInfo) throws Exception;

    /**
     * 海关缴款书导入
     *
     * @param pramsMap
     *            导入信息
     * @return 返回
     */
    String importCustoms(Map<String, Object> pramsMap);

    /**
     * 海关缴款书导入保存
     *
     * @param pramsMap
     *            导入信息
     * @param userInfo
     *            用户信息
     * @return 返回
     */
    String submitImportCustoms(Map<String, Object> pramsMap, UserInfo userInfo);

    /**
     * 待填
     *
     * @param page
     *            分页
     * @param entity
     *            条件
     * @param userInfo
     *            用户信息
     * @return 返回
     *
     */
    String exceptionDetail(Page<?> page, TdxCustomsRecordDTO entity, UserInfo userInfo);

    /**
     * 海关缴款书扫描录入
     *
     * @param pramsMap
     *            入参
     * @return 返回结果
     */
    String scanCollect(Map<String, Object> pramsMap);

    /**
     * 海关缴款书扫描录入确认
     *
     * @param pramsMap
     *            入参
     * @param userInfo
     *            用户信息
     * @return 返回结果
     */
    String scanCollectSubmit(Map<String, Object> pramsMap, UserInfo userInfo);
}
