package com.dxhy.erp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.service.impl.MpBaseServiceImpl;
import com.dxhy.erp.dao.SignRecordInvoiceStatisticsDao;
import com.dxhy.erp.entity.TdxRecordInvoiceStatistics;
import com.dxhy.erp.service.SignInvoiceStatisticsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 *
 *
 * 税率统计表业务处理实现类
 *
 * @author ld
 */
@Service("signInvoiceStatisticsService")
@Slf4j

public class SignInvoiceStatisticsServiceImpl
    extends MpBaseServiceImpl<SignRecordInvoiceStatisticsDao, TdxRecordInvoiceStatistics>
    implements SignInvoiceStatisticsService {

    @Override
    public void saveInvoiceStatistics(List<TdxRecordInvoiceStatistics> statisticsList, String dbName) {
        DynamicContextHolder.push(dbName + DbConstant.BUSINESS_WRITE);
        this.insertStatistics(statisticsList);
    }

    @Transactional(rollbackFor = Exception.class)
    public void insertStatistics(List<TdxRecordInvoiceStatistics> statisticsList) {
        this.saveBatch(statisticsList);
    }

    @Override
    public boolean deleteStatistics(String invoiceCode, String invoiceNo, String dbName) {
        DynamicContextHolder.push(dbName + DbConstant.BUSINESS_WRITE);

        return this.removeStatistics(invoiceCode, invoiceNo);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean removeStatistics(String invoiceCode, String invoiceNo) {
        try {

            int delete = baseMapper.delete(new QueryWrapper<TdxRecordInvoiceStatistics>()
                .eq("invoice_code", invoiceCode).eq("invoice_no", invoiceNo));
            return delete > 0;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

}
