package com.dxhy.manage.dao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.dxhy.manage.entity.SysDeptEntity;
import com.dxhy.manage.model.VersionModel;

/**
 * 部门管理
 *
 * @author jiaohongyang
 */
@Mapper
public interface SysDeptDao extends BaseMapper<SysDeptEntity> {
    /**
     * 查询集团信息
     *
     * @param company
     *            集团编码
     * @return 部门信息
     */
    List<SysDeptEntity> queryList(@Param("company") String company);

    /**
     * 查询集团信息
     *
     * @param company
     *            集团编码
     * @return 部门信息
     */
    List<SysDeptEntity> queryListUser(@Param("company") String company);

    /**
     * 查询集团信息
     *
     * @param company
     *            集团编码
     * @return 部门信息
     */
    List<SysDeptEntity> queryListParent(@Param("company") String company);

    /**
     * 查询子部门ID列表
     *
     * @param parentId
     *            上级部门ID
     * @return 部门id
     */
    List<Long> queryDetpIdList(@Param("parentId") Long parentId);

    /**
     * 查询部门信息
     *
     * @param parentId
     *            上级部门ID
     * @return 部门信息
     */
    List<SysDeptEntity> queryDeptParentId(@Param("parentId") Long parentId);

    /**
     * 查询部门信息
     *
     * @return
     */
    List<SysDeptEntity> queryDeptList();

    /**
     * 根据集团编码修改部分信息
     *
     * @param company
     *            company
     * @param aceKey
     *            aceKey
     * @param aceId
     *            aceId
     * @param upgradeStatus
     *            upgradeStatus
     * @param detailIntoWhereDb
     *            detailIntoWhereDb
     * @param detailIntoWhereTable
     *            detailIntoWhereTable
     * @param majorIntoWhereDb
     *            majorIntoWhereDb
     * @param majorIntoWhereTable
     *            majorIntoWhereTable
     * @param isCheckYesorno
     *            isCheckYesorno
     * @param isCollectYesorno
     *            isCollectYesorno
     * @param levelCode
     *            levelCode
     * @param isModel
     *            isModel
     */
    void updateDeptCompany(@Param("company") String company, @Param("aceKey") String aceKey,
        @Param("aceId") String aceId, @Param("upgradeStatus") String upgradeStatus,
        @Param("detailIntoWhereDb") String detailIntoWhereDb,
        @Param("detailIntoWhereTable") String detailIntoWhereTable, @Param("majorIntoWhereDb") String majorIntoWhereDb,
        @Param("majorIntoWhereTable") String majorIntoWhereTable, @Param("isCheckYesorno") String isCheckYesorno,
        @Param("isCollectYesorno") String isCollectYesorno, @Param("levelCode") String levelCode,
        @Param("isModel") String isModel);

    /**
     * 根据地区编码获取地址和版本号
     * 
     * @param area
     *            地区编码
     * @return 返回
     */

    VersionModel getVersion(String area);

}
